import urllib.request as url
import zipfile
import lxml
import lxml.etree
import os
import re
import csv
import datetime
import random
import os.path


# Set path to the folder you want to use.
# path = "/Users/win7/Desktop/data"

# Or use the folder this file is contained in
path = os.path.dirname(__file__)




### Download Kiva Data
print("Downloading data...")
data_loc = "http://s3.kiva.org/snapshots/kiva_ds_xml.zip"

# Since data set is relatively large, 1.2 GB, this will take a while. Also once 
# data set is downloaded one can comment out urlretrieve.
url.urlretrieve(data_loc, os.path.normpath(os.path.join(path, "kiva.zip")))

print("Extracting data...")
with zipfile.ZipFile(os.path.normpath(os.path.join(path, "kiva.zip")),"r") as zip_ref:
   zip_ref.extractall(path)



def ensureDirectoryExists(f): #http://stackoverflow.com/questions/273192/create-directory-if-it-doesnt-exist-for-file-write
  try:
    os.makedirs(f)
  except OSError:
    pass

ensureDirectoryExists( os.path.normpath(os.path.join(path, "tables/")) )



### Parse
print("Parsing data...")
count = 0



(cDirPath, acDirNames, acFileNames) = next(os.walk( os.path.normpath(os.path.join(path, "loans/")) ))

acAttributesToPull = ["id", "name", "borrowers/borrower/gender", "borrowers/borrower/pictured", "description/texts/en", "status", "funded_amount", "paid_amount", "terms/disbursal_amount", "loan_amount", "activity", "sector", "location/country", "location/town", "posted_date", "terms/disbursal_date", "terms/repayment_term", "funded_date", "paid_date"]
acListAttributesToPull = ["payments/payment/amount", "payments/payment/processed_date", "payments/payment/comment", "description/languages/language"]
attributes = acAttributesToPull + acListAttributesToPull
f = open(os.path.normpath(os.path.join(path, "tables/loans.csv")), "w", encoding='utf-8')
writer = csv.writer(f, lineterminator='\n')

# write header
writer.writerow(x.replace("/", "_") for x in attributes) 

for cFileName in acFileNames:
  cFilePath = os.path.normpath(os.path.join(path, "loans/" + cFileName)) 
  try:
    t = lxml.etree.parse( cFilePath )
  except lxml.etree.XMLSyntaxError:
    continue
  
  for e in t.xpath("//loan"):
    try:
      dValues = {}
      
      for cAttribute in acAttributesToPull:
        try:
          dValues[cAttribute] = re.sub('<[^>]*>', '', e.xpath( cAttribute )[0].text )
        except (IndexError, TypeError):
          dValues[cAttribute] = ""
      
      for cAttribute in acListAttributesToPull:
        dValues[cAttribute] = ""
        for x in e.xpath( cAttribute ):
          try:
            dValues[cAttribute] += x.text.strip("|") + "|"
          except (IndexError, AttributeError):
            dValues[cAttribute] += "|"
      
    except lxml.etree.XMLSyntaxError:
      continue
    
    writer.writerow( [ dValues[x] for x in attributes ] )
    count += 1
    if count % 2000 == 0:
      print(count)



f.close()





### Subset
print("Preparing data files...")
rows = []

variable_names = ['id', 'name', 'borrowers_borrower_gender', 'borrowers_borrower_pictured', 'description_texts_en', 'loan_amount', 'activity', 'sector', 'location_country', 'location_town', 'posted_date', 'terms_repayment_term', 'description_languages_language']
headings = ['id', 'name', 'gender', 'pictured', 'description', 'loan_amount', 'activity', 'sector', 'country', 'town', 'posted_date', 'repayment_term', 'languages']

loans = open(os.path.normpath(os.path.join(path, "tables/loans.csv")),encoding='utf-8')
reader = csv.DictReader(loans)
counter = 0
for row in reader:
    counter += 1 
    if counter % 5000 == 0:
      print(counter)
    
    if row['status'] != 'funded' and row['status'] != 'paid' and row['status'] != 'in_repayment' and row['status'] != 'defaulted':
      continue
    
    row['description_texts_en'] = row['description_texts_en'].replace('\n', ' ')
    
    #generate the days_until_funded variable:
    post = row['posted_date'][:10]
    fund = row['funded_date'][:10]
    days_until_funded = (datetime.datetime.strptime(fund, '%Y-%m-%d') - datetime.datetime.strptime(post, '%Y-%m-%d')).days
    if days_until_funded < 0:
      continue
    
    row['days_until_funded'] = str(days_until_funded)
    to_add = {x:row[x] for x in variable_names + ['days_until_funded']}
    
    #remove loans with missing values:
    if '' in list(to_add.values()):
      continue
    
    rows.append(to_add)

random.seed(4501)
random.shuffle(rows)

a_labeled = open(os.path.normpath(os.path.join(path, 'tables/loans_A_labeled.csv')), 'w',encoding='utf-8')
a_labeled_writer = csv.writer(a_labeled,lineterminator='\n')
a_labeled_writer.writerow([x for x in headings + ['days_until_funded']])
b_unlabeled = open(os.path.normpath(os.path.join(path, 'tables/loans_B_unlabeled.csv')), 'w',encoding='utf-8')
b_unlabeled_writer = csv.writer(b_unlabeled,lineterminator='\n')
b_unlabeled_writer.writerow([x for x in headings])
ab_labeled = open(os.path.normpath(os.path.join(path, 'tables/loans_AB_labeled.csv')), 'w',encoding='utf-8')
ab_labeled_writer = csv.writer(ab_labeled,lineterminator='\n')
ab_labeled_writer.writerow([x for x in headings + ['days_until_funded']])
b_labeled = open(os.path.normpath(os.path.join(path, 'tables/loans_B_labeled.csv')), 'w',encoding='utf-8')
b_labeled_writer = csv.writer(b_labeled,lineterminator='\n')
b_labeled_writer.writerow([x for x in headings + ['days_until_funded']])
c_labeled = open(os.path.normpath(os.path.join(path, 'tables/loans_C_labeled.csv')), 'w',encoding='utf-8')
c_labeled_writer = csv.writer(c_labeled,lineterminator='\n')
c_labeled_writer.writerow([x for x in headings + ['days_until_funded']])
c_unlabeled = open(os.path.normpath(os.path.join(path, 'tables/loans_C_unlabeled.csv')), 'w',encoding='utf-8')
c_unlabeled_writer = csv.writer(c_unlabeled,lineterminator='\n')
c_unlabeled_writer.writerow([x for x in headings])

size_of_B = (len(rows) - 100)/2 
truth = 0
for i, row in enumerate(rows):
  if i % 5000 == 0:    
    print(i)
  
  if i < 100:
    a_labeled_writer.writerow([row[n] for n in variable_names + ['days_until_funded']])
    ab_labeled_writer.writerow([row[n] for n in variable_names + ['days_until_funded']])
  elif i < size_of_B + 100:
    ab_labeled_writer.writerow([row[n] for n in variable_names + ['days_until_funded']])
    b_labeled_writer.writerow([row[n] for n in variable_names + ['days_until_funded']])
    b_unlabeled_writer.writerow([row[n] for n in variable_names])
  else:
    c_labeled_writer.writerow([row[n] for n in variable_names + ['days_until_funded']])
    c_unlabeled_writer.writerow([row[n] for n in variable_names])

a_labeled.close()
b_unlabeled.close()
ab_labeled.close()
b_labeled.close()
c_labeled.close()
c_unlabeled.close()

